/* -*- Mode: C; tab-width: 8; c-basic-offset: 4 -*- */
/* vim:set sw=4 ts=8: */
/*
 * This file is part of the MUSE Instrument Pipeline
 * Copyright (C) 2008-2014 European Southern Observatory
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 */

/*********************************************************************
 * E.S.O. - VLT project
 *
 * "@(#)"
 *
 * who       when        what
 * --------  ----------  ----------------------------------------------
 * apeconta  2012-10-08  added MUCLIPM_OVERSCAN definition and header parameter
 * gzins     2011-03-09  added MUCLIPM_NB_IFU definition
 * gzins     2011-03-09  suppressed muclipm_make_image_zero declaration
 * gzins     2011-03-05  re-declared muclipm_make_image_zero
 * gzins     2011-03-03  moved defines from C file and renamed them
 * sbn/ape   2010-03-15  updated and tested for MUSE
 * sdd-team  2009-12-15  created
 */

#ifndef MUCLIPM_MAKE_IMAGE_H
#define MUCLIPM_MAKE_IMAGE_H

/*-----------------------------------------------------------------------------
    Includes
 -----------------------------------------------------------------------------*/
#include <cpl.h> 

/*-----------------------------------------------------------------------------
    Declaration Block
 -----------------------------------------------------------------------------*/

#ifdef __cplusplus
extern "C" {
#endif

/*-----------------------------------------------------------------------------
                                   Defines
 -----------------------------------------------------------------------------*/
#define MUCLIPM_NB_SUBSLICERS      4 /* number of sub-slicers            */
#define MUCLIPM_NB_SPEC_PER_SLICE 75 /* number of pixels per slice       */
#define MUCLIPM_NB_SLICES         12 /* number of slices per sub-slicer  */
#define MUCLIPM_INTERSPEC          7 /* inter-spectrum distance in pixel */
#define MUCLIPM_NB_IFU            24 /* number of IFU                    */
#define MUCLIPM_OVERSCAN          32 /* overscan width in pixel          */
    
/*-----------------------------------------------------------------------------
    Procedures implemented by APE/SBN adapted to MUSE requirements
 -----------------------------------------------------------------------------*/
cpl_error_code  muclipm_make_image(const cpl_image    *raw_img,
                                   const cpl_image    *offset,
                                   const cpl_image    *flat,
                                   const cpl_image    *mask,
				   const long         *slice_xstart,
				   const long         *slice_xend,
                                   cpl_image         **out_img);

/*----------------------------------------------------------------------------*/
#ifdef __cplusplus
}   /* extern "C" */
#endif

#endif  /* MUCLIPM_MAKE_IMAGE_H */
