/* -*- Mode: C; tab-width: 2; indent-tabs-mode: nil; c-basic-offset: 2 -*- */
/* vim:set sw=2 sts=2 et cin: */
/* 
 * This file is part of the MUSE Instrument Pipeline
 * Copyright (C) 2005-2013 European Southern Observatory
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 */

/* This file was automatically generated */

#ifndef MUSE_QI_MASK_Z_H
#define MUSE_QI_MASK_Z_H

/*----------------------------------------------------------------------------*
 *                              Includes                                      *
 *----------------------------------------------------------------------------*/
#include <muse.h>
#include <muse_instrument.h>

/*----------------------------------------------------------------------------*
 *                          Special variable types                            *
 *----------------------------------------------------------------------------*/

/** @addtogroup recipe_muse_qi_mask */
/**@{*/

/*----------------------------------------------------------------------------*/
/**
  @brief   Structure to hold the parameters of the muse_qi_mask recipe.

  This structure contains the parameters for the recipe that may be set on the
  command line, in the configuration, or through the environment.
 */
/*----------------------------------------------------------------------------*/
typedef struct muse_qi_mask_params_s {
  /** @brief   Wavelength range (filter name) of the setup. Set this to "custom" to set up a new wavelength range using the wmin/wmax parameters. */
  int wrange;
  /** @brief   Wavelength range (filter name) of the setup. Set this to "custom" to set up a new wavelength range using the wmin/wmax parameters. (as string) */
  const char *wrange_s;

  /** @brief   Lower boundary of the wavelength range [Angstrom]. Only used when wrange=custom. */
  double wmin;

  /** @brief   Lower boundary of the wavelength range [Angstrom]. Only used when wrange=custom. */
  double wmax;

  /** @brief   IFU to handle. If set to 0, all IFUs are processed serially, which is the recommendation for this recipe, since only then all extensions end up in the same output file. */
  int nifu;

  char __dummy__; /* quieten compiler warning about possibly empty struct */
} muse_qi_mask_params_t;

#define MUSE_QI_MASK_PARAM_WRANGE_MUSE 1
#define MUSE_QI_MASK_PARAM_WRANGE_B 2
#define MUSE_QI_MASK_PARAM_WRANGE_V 3
#define MUSE_QI_MASK_PARAM_WRANGE_R 4
#define MUSE_QI_MASK_PARAM_WRANGE_I 5
#define MUSE_QI_MASK_PARAM_WRANGE_CUSTOM 6
#define MUSE_QI_MASK_PARAM_WRANGE_INVALID_VALUE -1

/**@}*/

/*----------------------------------------------------------------------------*
 *                           Function prototypes                              *
 *----------------------------------------------------------------------------*/
int muse_qi_mask_compute(muse_processing *, muse_qi_mask_params_t *);

#endif /* MUSE_QI_MASK_Z_H */
