/* -*- Mode: C; tab-width: 2; indent-tabs-mode: nil; c-basic-offset: 2 -*- */
/* vim:set sw=2 sts=2 et cin: */
/* 
 * This file is part of the MUSE Instrument Pipeline
 * Copyright (C) 2005-2013 European Southern Observatory
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 */

/* This file was automatically generated */

#ifndef MUSE_SKY_LSF_Z_H
#define MUSE_SKY_LSF_Z_H

/*----------------------------------------------------------------------------*
 *                              Includes                                      *
 *----------------------------------------------------------------------------*/
#include <muse.h>
#include <muse_instrument.h>

/*----------------------------------------------------------------------------*
 *                          Special variable types                            *
 *----------------------------------------------------------------------------*/

/** @addtogroup recipe_muse_sky_lsf */
/**@{*/

/*----------------------------------------------------------------------------*/
/**
  @brief   Structure to hold the parameters of the muse_sky_lsf recipe.

  This structure contains the parameters for the recipe that may be set on the
  command line, in the configuration, or through the environment.
 */
/*----------------------------------------------------------------------------*/
typedef struct muse_sky_lsf_params_s {
  /** @brief   Cut off the data below this wavelength after loading the pixel table(s). */
  double lambdamin;

  /** @brief   Cut off the data above this wavelength after loading the pixel table(s). */
  double lambdamax;

  /** @brief   Maximum number of minimization steps for the slice parameter fit. If set to zero, the slice parameter fit is omitted at all and the first guess is used. Note that this number is meant per degree of freedom, so the total number is 14 times higher. */
  int max_iterations;

  /** @brief   Output the pixel table after the LSF subtraction. */
  int store_subtracted;

  /** @brief   If specified, write an additional column containing the
          original data to the pixel table. */
  const char * orig;

  /** @brief   Use wavelength offset as slice dependent fit parameter. */
  int slice_fit_offset;

  /** @brief   Use the refraction index as slice dependent fit parameter. */
  int slice_fit_refraction;

  /** @brief   Use the slit width as fit parameter. */
  int slice_fit_slit_width;

  /** @brief   Use the bin width as fit parameter. */
  int slice_fit_bin_width;

  /** @brief   Polynomial order for the LSF width fit parameter. Use -1 to not fit the LSF width. */
  int slice_fit_lsf_width;

  /** @brief   Polynomial order for the H3 fit parameter. Use -1 to not fit H3. */
  int slice_fit_h3;

  /** @brief   Polynomial order for the H4 fit parameter. Use -1 to not fit H4. */
  int slice_fit_h4;

  /** @brief   Polynomial order for the H5 fit parameter. Use -1 to not fit H5. */
  int slice_fit_h5;

  /** @brief   Polynomial order for the H6 fit parameter. Use -1 to not fit H6. */
  int slice_fit_h6;

  char __dummy__; /* quieten compiler warning about possibly empty struct */
} muse_sky_lsf_params_t;


/**@}*/

/*----------------------------------------------------------------------------*
 *                           Function prototypes                              *
 *----------------------------------------------------------------------------*/
int muse_sky_lsf_compute(muse_processing *, muse_sky_lsf_params_t *);

#endif /* MUSE_SKY_LSF_Z_H */
